﻿// 
// $Id: CallbackHandler.cs 3982 2010-11-02 11:13:56Z unknown $
// 
// Zen UI Toolkit Copyright (c) 2010, Paul Clancy.  All rights reserved.
// 
// Redistribution and use in source and binary forms, with or without modification, are permitted provided 
// that the following conditions are met:
// 
// * 	Redistributions of source code must retain the above copyright notice, this list of 
// 	conditions and the following disclaimer.
// 
// * 	Redistributions in binary form must reproduce the above copyright notice, this list of 
// 	conditions and the following disclaimer in the documentation and/or other materials 
// 	provided with the distribution.
// 
// * 	Neither the name of AppTek nor the names of its contributors may be used to endorse 
// 	or promote products derived from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
// OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL 
// THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
// SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
// HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
// OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
// 

#region

using System;
using System.Globalization;
using System.IO;
using System.Reflection;
using System.Web;
using System.Web.SessionState;
using System.Web.UI;

#endregion

namespace Zen.Toolkit
{
    public class CallbackHandler : IHttpHandler, IRequiresSessionState
    {
        public const string ZEN_CALLBACK_MAGIC = "zenIsCallback";
        public const string ZEN_CALLBACK_METHOD_KEY = "zenMethod";
        public const string ZEN_CALLBACK_PARAMETER_KEY = "zenParams";

        #region IHttpHandler Members

        public void ProcessRequest(HttpContext context)
        {
            var page = (Page) PageParser.GetCompiledPageInstance(context.Request.Path, null, context);

            if (Z.IsZenCallback)
            {
                string callbackMethodName = context.Request[ZEN_CALLBACK_METHOD_KEY];
                MethodInfo[] pageMethods =
                    page.GetType().GetMethods(BindingFlags.Public | BindingFlags.Instance | BindingFlags.Static |
                                              BindingFlags.FlattenHierarchy | BindingFlags.NonPublic);

                MethodInfo targetMethod = null;

                foreach (MethodInfo methodInfo in pageMethods)
                {
                    if (methodInfo.Name == callbackMethodName)
                    {
                        // now check the args, should be "object sender, ZenEventArgs e"
                        ParameterInfo[] parameters = methodInfo.GetParameters();

                        if (parameters.Length == 2 && parameters[1].ParameterType == typeof (ZenEventArgs))
                        {
                            targetMethod = methodInfo;
                            break;
                        }
                    }
                }

                if (targetMethod == null)
                {
                    throw (new ZenHandlerNotFoundException(string.Format("Unable to find a handler called {0}",
                                                                         callbackMethodName)));
                }


                var e = new ZenEventArgs();

                foreach (string key in context.Request.Params.AllKeys)
                {
                    e.Parameters.Add(key, context.Request.Params[key]);
                }

                var args = new object[] {null, e};

                ZenEventResponse response = e.Response;
                try
                {
                    object rawResponse;

                    if (targetMethod.ReturnType != typeof (void))
                    {
                        rawResponse = targetMethod.Invoke(page, args);
                        response.Message = rawResponse.ToString();
                    }
                    else
                        targetMethod.Invoke(page, args);

                    var managerProxy = new ZenManager();
                    HttpContext.Current.Items[ZenManager.ZEN_MANAGER_CONTEXT_KEY] = managerProxy;


                    var writer = new StringWriter(CultureInfo.InvariantCulture);
                    using (var htmlWriter = new HtmlTextWriter(writer))
                    {
                        page.RenderControl(htmlWriter);
                    }

                    // assemble any scripting
                    //managerProxy.GenerateScript();

                    response.HtmlPayload = writer.ToString();
                    response.ScriptPayload = managerProxy.Script.BuildScript();

                    response.Success = true;
                }
                catch (TargetInvocationException ex)
                {
                    response.ScriptPayload = string.Empty;
                    response.Message = ex.InnerException.Message;
                    response.Success = false;
                }
                catch (Exception ex)
                {
                    response.ScriptPayload = string.Empty;
                    response.Message = ex.Message;
                    response.Success = false;
                }


                context.Response.ContentType = "text/json";
                context.Response.Write(JsonUtils.Serialize(response));
            }
            else
            {
                page.ProcessRequest(context);
            }
        }

        public bool IsReusable
        {
            get { return true; }
        }

        #endregion
    }
}